# -*- coding: utf-8 -*-
from django.http import FileResponse
from django.utils.translation import gettext_lazy as _
from rest_framework import status, viewsets
from rest_framework.decorators import action
from rest_framework.response import Response

from thx.api.mixins import ThuxModelMixin
from thx.role.core.views import ThuxRoleViewSetMixin

from ... import models
from . import serializers, filters, permissions

STATUS_CODES = ((0, _('disable')), (1, _('enable')),)


# COMMENT
class CommentViewSet(ThuxRoleViewSetMixin, ThuxModelMixin, viewsets.ModelViewSet):
    """
    CommentViewSet
    -----------

    :action list: Return the comment list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given comment.
        `Profiles`: `superuser`, `staff`

    :action light: Return the comment list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new comment instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update a comment instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch a comment instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Delete a comment instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.Comment.objects.all()
    serializer_class = serializers.CommentSerializer
    filter_backends = (filters.CommentFilterBackend,)
    permission_classes = (permissions.CommentPermission,)

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable a comment instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable a comment instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)


# ATTACHMENT
class AttachmentViewSet(ThuxRoleViewSetMixin, ThuxModelMixin, viewsets.ModelViewSet):
    """
    AttachmentViewSet
    -----------

    :action list: Return the attachment list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given attachment.
        `Profiles`: `superuser`, `staff`

    :action light: Return the attachment list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new attachment instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update a attachment instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch a attachment instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Delete a attachment instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.Attachment.objects.all()
    serializer_class = serializers.AttachmentSerializer
    filter_backends = (filters.AttachmentFilterBackend,)
    permission_classes = (permissions.AttachmentPermission,)

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable a attachment instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable a attachment instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)

    @action(methods=["get"], detail=True, url_name="download")
    def download(self, request, *args, **kwargs):
        instance = self.get_object()
        try:
            response = FileResponse(
                open(instance.attachment.path, 'rb'),
                as_attachment=True,
                filename=f'{instance.attachment}'
            )
            response['Content-Disposition'] = 'attachment; filename="{}"'.format(instance.attachment.name)
            return response
        except FileNotFoundError as fnf:
            return Response(fnf.strerror, status=status.HTTP_404_NOT_FOUND)
